import { logger } from "./logger.js";
import { getTimers } from "./timer.repository.js";
import { getStatus, pauseTimer, resumeTimer, deleteTimer, flipTimers, subscribe } from './timer.loop.js';
import { TimerError } from "./error.js";
import { showAddTimer } from './add-timer-screen.js';

const templatePath = `modules/yet-another-timer/templates/timer.hbs`;

/**
 * Log the timer
 * @param {Timer} timerInstance The timer to display.
 * @returns {void}
 */
export function logTimer(timerInstance) {
  logger.group(`Timer ${timerInstance.name}`);
  logger.log(`Logging the timer:`, timerInstance);
  logger.log(`Timer: ${timerInstance.name}`);
  logger.log(`  Initial: ${timerInstance.duration}`);
  logger.log(
    `  Remaining: ${timerInstance.durationSeconds - timerInstance.elapsedTimeSeconds
    }`,
  );
  logger.log(`  Paused: ${timerInstance.isPaused}`);
  logger.groupEnd(`Timer ${timerInstance.name}`);
}

logger.debug(templatePath);

export class TimerScreen extends FormApplication {
  static get defaultOptions() {
    const defaults = super.defaultOptions;

    const overrides = {
      height: "auto",
      resizable: true,
      id: "yet-another-timer",
      template: templatePath,
      title: game.i18n?.localize("YAT.title") || "Yet another timer",
    };
    return Object.assign({}, defaults, overrides);
  }

  getData() {
    return {
      timers: getTimers(),
      config: {
        isPaused: getStatus()
      }
    };
  }

  activateListeners(html) {
    logger.debug('Activating listeners...');
    html.on('click', "[data-action]", this.#handleClick.bind(this));
    logger.debug('Activated.');
    // also make sure we're always active
    subscribe(() => {
      this.render();
    });
  }

  #handleClick(event) {
    logger.debug('**click**', event);
    const el = $(event.currentTarget);
    const action = el.data().action;
    const timerName = el.parents('[data-timer-name]')?.data()?.timerName;
    console.log(el);
    logger.debug('timerName and action', timerName, action);

    switch(action) {
      case 'flip-timers':
        flipTimers();
        break;
      case 'pause':
        pauseTimer(timerName);
        break;
      case 'resume':
        resumeTimer(timerName);
        break;
      case 'delete':
        deleteTimer(timerName);
        break;
      case 'add-timer':
        showAddTimer()
        break;
      default:
        logger.error(`Unknown timer action: ${action}`);
        throw new TimerError(`Unknown timer action: ${action}`);
    }
    this.render();
  }
}

function formatRemainingTime(durationSeconds, elapsedTimeSeconds) {
  const total = Number(durationSeconds) || 0;
  const elapsed = Number(elapsedTimeSeconds) || 0;

  const remainingSeconds = Math.max(0, total - elapsed);

  // Calculate minutes and seconds
  const minutes = Math.floor(remainingSeconds / 60);
  const seconds = Math.floor(remainingSeconds % 60); // Use Math.floor to handle potential floating point inaccuracies

  // Pad with leading zeros if needed
  const formattedMinutes = String(minutes).padStart(2, "0");
  const formattedSeconds = String(seconds).padStart(2, "0");

  return `${formattedMinutes}:${formattedSeconds}`;
}
Handlebars.registerHelper('formatRemainingTime', formatRemainingTime);
