import { logger } from "./logger.js";
import { TimerError } from "./error.js";

const MAX_DURATION = 60 * 60 * 24; // one day, let's say.

/**
  * @typedef {string} TimeString
  * @description A string-wrapped formatted duration:
  * 1. Seconds only: ("123")
  * 2. Minutes and seconds as "MM:SS" (leading zeros)
  * 3. Hours, minutes, seconds as "HH:MM:SS".
  * @example "125" // 125 seconds
  * @example "13:22" // 13 minutes 22 seconds
  * @example "02:04:06" // 2 hours, 4 minutes, 6 seconds
  */

/**
  * A Timer instance
  * @typedef {Object} Timer
  * @property {TimeString} duration - total duration time
  * @property {number} elapsedTimeSeconds - time we've ran so far
  * @property {boolean} isPaused - flag indicating if the timer is paused.
  */


const timerMap = new Map();

/**
  * Lists all timer names and states
  * @returns {Array<Timer>}
  */ 
export function getActiveTimers() {
  logger.debug(`Getting active timers...`);
  return Array.from(timerMap)
    .map(([_, timer]) => timer)
    .filter(timer => !timer.isPaused);
}

export function getTimers() {
  return Array.from(timerMap)
    .map(([_, timer]) => timer)
}


export function getTimer(timerName = '') {
  if (!timerMap.has(timerName)) {
    throw new TimerError('No such timer');
  }
  return timerMap.get(timerName); 
}

export function deleteTimer(timerName = '') {
  if (timerMap.has(timerName)) {
    timerMap.delete(timerName);
  } else {
    throw new TimerError(`Trying to delete non-existent timer ${timerName}.`);
  }
}

/**
  * Create a new timer.
  * @param {TimeString} [durationString='01:00'] - the timer duration
  * @param {TimeString} [name='<next serial name>'] - name for the timer instance
  * @param {boolean} [startNow=false] - start immediately
  * @returns {(string | TimerError)} - returns the timerId or timer.
  */ 
export function createTimer(durationString = '01:00', name = '', startNow = false) {
  let timerName = name;
  if (timerName === '') {
    timerName = getNextName();
  }

  assertValidName(timerName);
  assertValidDuration(durationString);

  timerMap.set(timerName, {
    name: timerName,
    duration: durationString,
    elapsedTimeSeconds: 0,
    isPaused: !startNow,
    durationSeconds: getDurationInSeconds(durationString),
  });
  return timerName;
}

function assertValidName(name) {
  if (timerMap.has(name)) {
    throw new TimerError(`Timer with ${name} already exists`);
  }
}

export function getNextName(prefix = 0) {
  const name = `timer-${prefix}-${timerMap.size}`
  if (timerMap.has(name)) {
    return getNextName(prefix + 1);
  }
  return name;
}

const secondsRe = /^\d+$/;
const mmssRe = /^(\d{2}):(\d{2})$/;
const hhmmssRe = /^(\d{2}):(\d{2}):(\d{2})$/;
const minutesOrHoursRe = /(\d{2}):(\d{2})$/;

function assertValidDuration(durationString) {
  if (!durationString) {
    throw new TimerError('Please provide a duration string');
  }
  const parts = minutesOrHoursRe.exec(durationString);
  if (parts) {
    if (parts[1] >= 60) {
      throw new TimerError('Minutes must be between 0 and 59.');
    }
    if (parts[2] >= 60) {
      throw new TimerError('Seconds must be between 0 and 59.');
    }
  } else if (!secondsRe.test(durationString)) {
    throw new TimerError('Must be formatted "<seconds>", "mm:ss" or "hh:mm:ss" format');
  }
}

function getDurationInSeconds(durationString) {
  let duration;
  if (secondsRe.test(durationString)) {
    duration = parseInt(durationString, 10);
  } else if (mmssRe.test(durationString)) {
    const parts = mmssRe.exec(durationString);
    duration = parseInt(parts[0], 10) * 60 + parseInt(parts[1], 10);
  } else if (hhmmssRe.test(durationString)) {
    const parts = hhmmssRe.exec(durationString);
    duration = parseInt(parts[0], 10) * 24 * 60 + parseInt(parts[1], 10) * 60 + parseInt(parts[2], 10);
  } else {
    throw new TimerError(`Duration "${durationString}" not calculable`);
  }

  if (duration > MAX_DURATION) {
    logger.warn(`Duration greater then max duration, capping the value to ${MAX_DURATION}.`);
    return MAX_DURATION;
  } else {
    return duration;
  }
}
